﻿using Eco.Gameplay.Players;
using Eco.Gameplay.Skills;
using Eco.Gameplay.Systems.Chat;
using Eco.Gameplay;
using Eco.Mods.TechTree;

using System;
using System.IO;
using System.Text;

namespace ResetSkills
{
    public class ResetSkillsPlugin : IChatCommandHandler
    {
        public const int MaxResets = 3;
        public const string PluginDirectoryPath = "./ResetSkillsPlugin";

        [ChatCommand(ChatAuthorizationLevel.User)]
        public static void ResetSkills(User sender)
        {
            ResetPlayerSkills(sender);
        }

        private static void ResetPlayerSkills(User user)
        {
            int resets = GetPlayerResets(user);

            if(resets > 0)
            {
                int theoreticalRefund = 0;
                int realRefund;
                string numberString = "points";

                foreach(Skill skill in user.Skillset.Skills)
                {
                    theoreticalRefund += skill.TotalPointSpent;
                    skill.ForceSetLevel(0);
                }

                realRefund = theoreticalRefund - GetPlayerRefundDifference(user);
                user.UseXP(-realRefund);

                SetPlayerResets(user, resets - 1);
                SetPlayerRefundDifference(user, theoreticalRefund);

                if(realRefund == 1)
                {
                    numberString = "point";
                }

                user.Player.SendTemporaryMessageLoc("Poof! Your skills are gone. (+" + realRefund + " skill " + numberString + ")");
            }
            else
            {
                user.Player.SendTemporaryMessageLoc("You can not reset your skills anymore.");
            }
        }

        private static void CheckPluginDirectory()
        {
            if (!Directory.Exists(PluginDirectoryPath))
            {
                Directory.CreateDirectory(PluginDirectoryPath);
            }
        }

        private static void SetPlayerResets(User user, int resets)
        {
            CheckPluginDirectory();

            File.WriteAllText(Path.Combine(PluginDirectoryPath, user.Player.FriendlyName + ".resets"), resets.ToString());
        }

        private static int GetPlayerResets(User user)
        {
            string path = Path.Combine(PluginDirectoryPath, user.Player.FriendlyName + ".resets");

            CheckPluginDirectory();

            if(File.Exists(path))
            {
                string text = File.ReadAllText(path);

                return Convert.ToInt32(text);
            }

            return MaxResets;
        }

        private static void SetPlayerRefundDifference(User user, int refunded)
        {
            CheckPluginDirectory();

            File.WriteAllText(Path.Combine(PluginDirectoryPath, user.Player.FriendlyName + ".difference"), refunded.ToString());
        }

        private static int GetPlayerRefundDifference(User user)
        {
            string path = Path.Combine(PluginDirectoryPath, user.Player.FriendlyName + ".difference");

            CheckPluginDirectory();

            if (File.Exists(path))
            {
                string text = File.ReadAllText(path);

                return Convert.ToInt32(text);
            }

            return 0;
        }
    }
}
